/* beeper.c - a tool for emitting a sound when newlines occur in stdin.  */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>
#include <time.h>

void usage () __attribute__ ((noreturn));
void argerr () __attribute__ ((noreturn));

static char *program_name;

/* getopt flags.  */
static int show_help;
static int show_version;

/* getopt long options equivalences.  */
static struct option long_options[] = {
  {"lines", required_argument, NULL, 'n'},
  {"delay", required_argument, NULL, 'd'},
  {"max", required_argument, NULL, 'm'},
  {"version", no_argument, NULL, 'V'},
  {"help", no_argument, &show_help, 1},
  {0, 0, 0, 0}
};

/* For --lines option.  */

static unsigned int newline_limit;

/* For --max option.  */

static unsigned int maxbeep;

/* For --delay option.  */

static int delay_active;
double sec;
static struct timespec timesleep;


void
usage (status)
     int status;
{
  if (status != 0)
    {
      fprintf (stderr, "Usage: %s [OPTION]\n", program_name);
      fprintf (stderr, "Try `%s --help' for more information.\n",
	       program_name);
    }
  else
    {
      printf ("Usage: %s [OPTION]\n", program_name);
      printf ("\
Beep when a newline occur in standard input.\n\
\n\
  -n, --lines=NUM           beep for each NUM lines\n\
  -m, --max=NUM             no more of NUM beep\n\
  -d, --delay=NUM[.n]       delay for NUM[.n] seconds\n\
      --help                display this help and exit\n\
  -V, --version             print version information and exit\n");

      printf ("\nReport bugs to <%s>.\n", PACKAGE_BUGREPORT);
    }
  exit (status);

}

void
arg_err (q_arg, errno, arg)
     const char *q_arg;
     unsigned int errno;
     const char *arg;
{
#define ERROR_INVALID_NUMBER	0
#define ERROR_BIG_NUMBER	1
  const char *errlist[] = {
    "isn't valid number for",
/*    "is greater than",*/
  };

  fprintf (stderr, "%s: '%s' %s %s\n", program_name, q_arg, errlist[errno],
	   arg);
  exit (2);
}

/* A xnanosleep() interface arrangement for my small program. Original */
/* written by Paul Eggert. Details in GNU Coreutils source.  */

void
double2timespec (double seconds, struct timespec *ts_sleep)
{
  double ns;
  ts_sleep->tv_sec = seconds;
  ns = 1e9 * (seconds - ts_sleep->tv_sec);
  ts_sleep->tv_nsec = ns;
  ts_sleep->tv_nsec += (ts_sleep->tv_nsec < ns);

  if (1000000000 <= ts_sleep->tv_nsec)
    {
      time_t t = ts_sleep->tv_sec + 1;
      ts_sleep->tv_sec = t;
      ts_sleep->tv_nsec -= 1000000000;
    }
}

void
beep ()
{
  register int c;
  register unsigned int newline = newline_limit;
  unsigned int maxbeepset = maxbeep;

  while ((c = getc (stdin)) != EOF)
    {
      putchar (c);
      if (c == '\n')
	{
	  if (newline)
	    --newline;
	  else
	    {
	      if (maxbeep || !maxbeepset)
		fputc ('\007', stderr);

	      if (delay_active)
		nanosleep (&timesleep, NULL);

	      if (maxbeepset && maxbeep)
		maxbeep--;

	      if (!newline && newline_limit)
		newline = newline_limit;
	    }
	}
    }
}

int
main (int argc, char *argv[])
{
  int opt;
  extern char *optarg;
  extern int optind;

  program_name = argv[0];
  if (program_name && strrchr (program_name, '/'))
    program_name = strrchr (program_name, '/') + 1;

  setlocale (LC_NUMERIC, "");

/* bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE)  */

  while ((opt =
	  getopt_long (argc, argv, "Vn:d:m:", long_options, NULL)) != EOF)
    switch (opt)
      {
      case 0:
	break;
      case 'V':
	show_version = 1;
	break;
      case 'n':
	if (optarg)
	  {
	    if ((newline_limit = atoi (optarg)) <= 0)
	      arg_err (optarg, ERROR_INVALID_NUMBER, "-n");
	    newline_limit--;
	  }
	break;
      case 'd':
	if (optarg)
	  {
	    sec = strtod (optarg, NULL);
	    if (sec >= 0)
	      double2timespec (sec, &timesleep);
	    else
	      arg_err (optarg, ERROR_INVALID_NUMBER, "-d");
	    delay_active = sec > 0;
	  }
	break;
      case 'm':
	if (optarg)
	  maxbeep = atoi (optarg);
	break;
      default:
	usage (2);
	break;
      }

  if (show_version)
    {
      printf ("%s %s\n", PACKAGE_NAME, VERSION);
      exit (0);
    }

  if (show_help)
    usage (0);

  setvbuf (stdin, NULL, _IOLBF, 0);
  setvbuf (stdout, NULL, _IOLBF, 0);

  beep ();

  return 0;
}
